@@ -1,5 +1,7 @@
 import win32com.client
 import pyscreeze
+import win32gui
+from PIL import ImageGrab
 
 import subprocess
 import time
@@ -23,13 +25,16 @@ class SapGui:
             self.SapGuiAuto = None
             return
 
-        # connection = application.Children(0)
         self.connection = self.application.OpenConnection(nombre_conexion, True)
         if not type(self.connection) == win32com.client.CDispatch:
             self.application = None
             self.SapGuiAuto = None
             return
 
+        if len(self.connection.Children) == 0:
+            self.connection.CloseConnection()
+            raise Exception('No se encuentra sesión, hay que habilitar scripting')
+
         self.session = self.connection.Children(0)
         if not type(self.session) == win32com.client.CDispatch:
             self.connection = None
@@ -43,17 +48,32 @@ class SapGui:
         self.session.findById("wnd[0]").sendVKey(0)
         self.session.findById("wnd[0]").maximize()
 
-        if self.session.findById('wnd[1]', False):
-            self.session.findById("wnd[1]/usr/radMULTI_LOGON_OPT2").select()
-            self.session.findById("wnd[1]/tbar[0]/btn[0]").press()
+        # más de una sesión abierta, pulsamos continuar sin cerrar sesión
+        try:
+            if self.session.findById('wnd[1]', False):
+                self.session.findById("wnd[1]/usr/radMULTI_LOGON_OPT2").select()
+                self.session.findById("wnd[1]/tbar[0]/btn[0]").press()
+        except Exception:
+            pass
+
+        # intento fallido de contraseña
+        try:
+            if self.session.findById('wnd[1]', False):
+                self.session.findById("wnd[1]/tbar[0]/btn[0]").press()
+        except Exception:
+            pass
 
     def logout(self):
         self.session.findById("wnd[0]").close()
-        try:
-            self.session.findById("wnd[1]/usr/btnSPOP-OPTION1").press()
-        except Exception:
+        if self.session.findById("wnd[3]/usr/btnSPOP-OPTION1", False) is not None:
+            self.session.findById("wnd[3]/usr/btnSPOP-OPTION1").press()
+
+        elif self.session.findById("wnd[2]/usr/btnSPOP-OPTION1", False) is not None:
             self.session.findById("wnd[2]/usr/btnSPOP-OPTION1").press()
 
+        else:
+            self.session.findById("wnd[1]/usr/btnSPOP-OPTION1").press()
+
     def close(self):
         self.connection = None
         self.application = None
@@ -67,3 +87,17 @@ class SapGui:
             return True
         except Exception:
             return False
+
+    def save_screenshot(self, screenshot_name):
+        try:
+            hwnd = win32gui.FindWindowEx(0, 0, 0, self.session.findById("wnd[0]").Text)
+            shell = win32com.client.Dispatch("WScript.Shell")
+            shell.SendKeys('%')  # evita que salte error en metodo setforegroundwindow
+            win32gui.SetForegroundWindow(hwnd)
+            time.sleep(0.5)
+            bbox = win32gui.GetWindowRect(hwnd)
+            img = ImageGrab.grab(bbox)
+            img.save(screenshot_name)
+            return True
+        except Exception:
+            return False
